package xyz.lychee.lagfixer.libs.adventure.inventory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class BookImpl implements Book {
   private final Component title;
   private final Component author;
   private final List<Component> pages;

   BookImpl(final @NotNull Component title, final @NotNull Component author, final @NotNull List<Component> pages) {
      this.title = (Component)Objects.requireNonNull(title, "title");
      this.author = (Component)Objects.requireNonNull(author, "author");
      this.pages = Collections.unmodifiableList((List)Objects.requireNonNull(pages, "pages"));
   }

   public @NotNull Component title() {
      return this.title;
   }

   public @NotNull Book title(final @NotNull Component title) {
      return new BookImpl((Component)Objects.requireNonNull(title, "title"), this.author, this.pages);
   }

   public @NotNull Component author() {
      return this.author;
   }

   public @NotNull Book author(final @NotNull Component author) {
      return new BookImpl(this.title, (Component)Objects.requireNonNull(author, "author"), this.pages);
   }

   public @NotNull List<Component> pages() {
      return this.pages;
   }

   public @NotNull Book pages(final @NotNull List<Component> pages) {
      return new BookImpl(this.title, this.author, new ArrayList((Collection)Objects.requireNonNull(pages, "pages")));
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("title", this.title), ExaminableProperty.of("author", this.author), ExaminableProperty.of("pages", this.pages));
   }

   public boolean equals(final Object o) {
      if (this == o) {
         return true;
      } else if (!(o instanceof BookImpl)) {
         return false;
      } else {
         BookImpl that = (BookImpl)o;
         return this.title.equals(that.title) && this.author.equals(that.author) && this.pages.equals(that.pages);
      }
   }

   public int hashCode() {
      int result = this.title.hashCode();
      result = 31 * result + this.author.hashCode();
      result = 31 * result + this.pages.hashCode();
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   static final class BuilderImpl implements Book.Builder {
      private Component title = Component.empty();
      private Component author = Component.empty();
      private final List<Component> pages = new ArrayList();

      @NotNull
      public Book.@NotNull Builder title(final @NotNull Component title) {
         this.title = (Component)Objects.requireNonNull(title, "title");
         return this;
      }

      @NotNull
      public Book.@NotNull Builder author(final @NotNull Component author) {
         this.author = (Component)Objects.requireNonNull(author, "author");
         return this;
      }

      @NotNull
      public Book.@NotNull Builder addPage(final @NotNull Component page) {
         this.pages.add((Component)Objects.requireNonNull(page, "page"));
         return this;
      }

      @NotNull
      public Book.@NotNull Builder pages(final @NotNull Collection<Component> pages) {
         this.pages.addAll((Collection)Objects.requireNonNull(pages, "pages"));
         return this;
      }

      @NotNull
      public Book.@NotNull Builder pages(final @NotNull Component... pages) {
         Collections.addAll(this.pages, pages);
         return this;
      }

      public @NotNull Book build() {
         return new BookImpl(this.title, this.author, new ArrayList(this.pages));
      }
   }
}
